<?php
/*
    Plugin Name: Embedder Plugin
    Plugin URI: http://embedder.englishmike.net/
    Description: A single plugin for managing all your embeds.
    Version: 0.1
    Author: English Mike
    Author URI: http://englishmike.net
*/

/*  Copyright 2008  Michael J. Walker  (email : embedder@englishmike.net)

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
*/

// KNOWN BUGS AND ISSUES

global $wpdb;

define('EMB_DEBUG', true);
define('EMB_PLUGIN_VERSION', '0.1');
define('EMB_ADVANCED_OPTIONS', 0);
define('EMB_PLUGIN_FILE', 'embedder/embedder.php');
define('EMB_TABLE', $wpdb->prefix.'emb_embeds');
define('EMB_PAGEPREFIX', 'tools_page_');
define('EMB_PAGEID', 'embed-admin');
define('EMB_PAGENAME', 'tools.php?page='.EMB_PAGEID);
define('EMB_OS_WIN', substr(PHP_OS, 0, 3) == 'WIN');

register_activation_hook(__FILE__, 'emb_plugin_activate');
register_deactivation_hook(__FILE__, 'emb_plugin_deactivate');

add_action('admin_menu', 'emb_add_admin_page');
emb_add_embeds();
add_filter('the_content', 'emb_add_post_embeds');
add_action( 'contextual_help', 'emb_add_help_text', 10, 3);

function emb_add_post_embeds($content) {
    $field = get_post_custom();
    foreach ($field as $key => $value) {
       $key = strtolower(trim($key));
       if (strpos($key, '[') == 0 && strpos($key, ']') == strlen($key) - 1) {
           add_shortcode(trim($key, '[] '), 'emb_do_post_embed');
       }
    }
    return $content;
}

function emb_do_post_embed($attrs = null, $content = null, $name) {
    $list = get_post_custom_values('['.strtolower($name).']');
    $value = $list[0];
    if (!empty($attrs) || !empty($content)) {
        $value = emb_process_attributes($value, $attrs, $content);
    }
    return $value;
}

function emb_add_embeds() {

    global $wpdb, $emb_codes;
    emb_trace("emb_add_embeds");
    $entries = $wpdb->get_results("SELECT embed, value FROM ".EMB_TABLE);
    if (count($entries) != 0) {
        foreach ($entries as $entry) {
            add_shortcode($entry->embed, 'emb_do_embed');
            $emb_codes[$entry->embed] = $entry->value;
            emb_trace("emb_add_embeds: $entry->embed");
        }
    }
}

function emb_do_embed($attrs = null, $content = null, $name) {

    global $wpdb, $emb_codes;
    emb_trace("emb_do_embed: $name");
    $name = strtolower(trim($name));
    // $value = 'NOT FOUND';
    // $entries = $wpdb->get_results("SELECT value FROM ".EMB_TABLE." WHERE embed = '$name'");
    if (!empty($emb_codes) && !empty($emb_codes[$name])) {
        $value = html_entity_decode($emb_codes[$name]);
        if (!empty($attrs) || !empty($content)) {
            $value = emb_process_attributes($value, $attrs, $content);
        }
    }
    return $value;
}

function emb_process_attributes($value, $attrs, $content) {

    if (!empty($attrs)) {
        $var = array_keys($attrs);
        $val = array_values($attrs);
        for ($i = 0; $i < sizeof($var); $i++) {
            $var[$i] = strtolower("/%$var[$i]%/i");
        }
    }
    $var[] = "/%content%/";
    $val[] = $content;

    $var[] = "/%!!/";
    $val[] = "%";
    return preg_replace($var, $val, $value);
}


// Only add this action if we're on the index management page.
// Avoids function getting called unnecessarily.
if ($_GET['page'] == 'embed-admin') {
    add_action('init', 'emb_process_requests');
}

/**
 * Add the manage embeds admin page to the admin menu, and if there
 * was a problem installing this version of Embedder, then hook into
 * the admin_notices action to display the error message.
 */
function emb_add_admin_page() {
    add_management_page('Manage Embeds', 'Embeds', 8, 'embed-admin', 'emb_display_admin_page');
    $error = get_option('emb_plugin_error');
    if (!empty($error)) {
        add_action('admin_notices', 'emb_admin_notices');
    }
}

/**
 * Set up all the available options as keys to an associative array with the values being
 * the description of what each option does. 
 *
 * @param $keys if true, create an array of the keys (option names) only
 * @return array of options
 */
function emb_get_options($keys = false) {
    $options = array();
    if ($keys) {
        $options = array_keys($options);
    }
    return $options;
}

/**
 * Called when the plugin is activated. Create an empty embed table in the WordPress 
 * database, if one does not already exist.  If we are upgrading then we create a 
 * wp_emb_upgrade table first and then, if the table is successfully upgraded then
 * we rename it to wp_emb_embeds.  This prevents data loss if something goes badly wrong
 * during the upgrade.
 */
function emb_plugin_activate() {
    emb_trace("fn:emb_plugin_activate : start");
    
    include_once(ABSPATH.'/wp-admin/includes/upgrade.php');
    global $wpdb;

    // Check to see if we need to upgrade the database table.
    if (emb_is_table_present() && !emb_is_compatible_version()) {
        emb_trace("upgrading...");
        $upgrade = '_upgrade';    // Suffix of new table during the upgrade
        $old = '_old';            // Suffix of old table during the upgrade
    	$entries = $wpdb->get_results("SELECT * FROM ".EMB_TABLE);
    }

    // Create the Embed database table.
    $options = emb_get_options(true);
    $sql = "CREATE TABLE ".EMB_TABLE.$upgrade." ( embed varchar(20) NOT NULL, description text, value text";

    if (!empty($options)) {
       $sql .= ", options set(";
       for ($i = 0; $i < count($options); $i++) {
           $sql .= ($i > 0 ? ", " : "")."'".$options[$i]."'";                                       
       }   
       $sql .= ")";
    }
    $sql .= ", PRIMARY KEY(embed))";

    // Create the table
    $created = maybe_create_table(EMB_TABLE.$upgrade, $sql);
    
    // Only create the table if old does not already exist.
    if ($created) {
        if (!empty($upgrade)) {
            emb_trace("upgrading...");
            // Put upgrading logic here
        }
        emb_trace("fn:emb_plugin_activate : end");
    } else {
        $error = !empty($upgrade) ? 'create-table-failed' : 'fresh-create-table-failed';
        emb_trace("fn:emb_plugin_activate : error : ".$sql);
    }

    // Perform the final step of the upgrade if everything has gone well so far, by
    // renaming the upgraded database table to wp_emb_embeds.
    if (!empty($upgrade) && empty($error)) {
        $rc = $wpdb->query("RENAME TABLE ".EMB_TABLE." TO ".EMB_TABLE.$old.', '.EMB_TABLE.$upgrade." TO ".EMB_TABLE);
        if ($rc !== false) {
            $rc = $wpdb->query("DROP TABLE IF EXISTS ".EMB_TABLE.$old);
        } else {
            $error = 'rename-table-failed';
        }
    }

    // If there was an error then remove the extra tables created during
    // the upgrade and save the error to be display during the admin-notices hook.
    if (!empty($error)) {
        if (!empty($upgrade)) {
            $wpdb->query("DROP TABLE IF EXISTS ".EMB_TABLE.$upgrade);
            $wpdb->query("DROP TABLE IF EXISTS ".EMB_TABLE.$old);
        }
        add_option('emb_plugin_error', $error);
    }
    update_option("emb_plugin_version", EMB_PLUGIN_VERSION);
}

/**
 * Called when the plugin is deactivated.  Currently does nothing.
 */
function emb_plugin_deactivate() {
    emb_trace("fn:emb_plugin_deactivate : start");
    emb_trace("fn:emb_plugin_deactivate : end");
}

/**
 * Called when the plugin is uninstalled.  Removes the database table
 * and all the Embedder options.  
 */
function emb_plugin_uninstall() {
    require_once(ABSPATH.'wp-admin/includes/plugin.php');
    emb_trace("fn:emb_plugin_uninstall : start");
    global $wpdb;
    deactivate_plugins(EMB_PLUGIN_FILE);
    delete_option('emb_plugin_version');
    delete_option('emb_plugin_error');    
    $wpdb->query("DROP TABLE IF EXISTS ".EMB_TABLE);
    emb_trace("fn:emb_plugin_uninstall : end");
}

/**
 * Check for the first compatible level of the database. 
 * This is currently version 0.7.1 when the CREATE TABLE statement was modified.
 * 
 * NOTE: The following is the way to test for a field change (if there are no option changes)
 *       $compatible = $wpdb->query("SHOW COLUMNS FROM ".EMB_TABLE." LIKE 'itemcache'");
 *
 *       The following is the way to test for an option change (i.e. just a new option was added:
 *       $compatible = false;
 *       $results = $wpdb->get_col("SHOW COLUMNS FROM ".EMB_TABLE." LIKE 'options'", 1);    
 *       if (!empty($results)) {
 *          $compatible = strpos($results[0], 'child-cats') !== false;         
 *       }
 *  
 * @return true if this is a compatible version of the database
 */
function emb_is_compatible_version() {
    global $wpdb;
    $version = get_option('emb_plugin_version');
    $compatible = version_compare($version, '0.1', '>=');
    return $compatible;
}

/**
 * Detect the wp_emb_embeds table to make sure it is present.
 *
 * @return true if the table is found in the blog's database
 */
function emb_is_table_present() {
    global $wpdb;
    return $wpdb->query("DESCRIBE ".EMB_TABLE);
}

/**
 * Process the action requested by the user, then redirect to the main indexes admin page.  
 * If no action is specified, then skip all processing.
 */
function emb_process_requests() {

    emb_trace('POST keys = '.implode('|', array_keys($_POST)));
    emb_trace('POST vals = '.implode('|', $_POST));
    emb_trace(' GET keys = '.implode('|', array_keys($_GET)));
    emb_trace(' GET vals = '.implode('|', $_GET));
    
    emb_trace('--- init --------------------------------------------');
    emb_trace('fn:emb_process_requests : start');
    global $emb_req;
    
    $req = new emb_request();
        
    emb_trace('fn:emb_process_requests : action = '.$req->action);
    if (!empty($req->action) && ($req->action == 'embed-add-entry' || $req->action == 'embed-update-entry' || $req->action == 'embed-rename-entry' 
                              || $req->action == 'embed-copy-entry' || $req->action == 'embed-delete-entry' || $req->action == 'cancel' 
                              || $req->action == 'embed-uninstall-plugin')) {

        if ($req->action == 'embed-update-entry' || $req->action == 'embed-add-entry' 
         || $req->action == 'embed-rename-entry' || $req->action == 'embed-copy-entry') {

            // Collect the parameters from the POST data returned from the user.
            $req->set_vars_from_post();
        
            if (empty($req->error_field)) {
                emb_trace("fn:emb_process_requests : no errors found");
                $req->escape_vars();
            }
        
        } else if ($req->action == 'embed-delete-entry') {
            // Collect the parameters from the GET data returned from the user and
            // fill the rest in from the database entry for the index.
            $req->set_vars_from_get();
        }
    
        if (empty($req->error_field)) {
    
            global $wpdb;
            $msgno = 0;
            $wpdb->show_errors();

            if ($req->action == 'embed-update-entry') {
                 
                // Update the settings for the index being edited.
                emb_trace("fn:emb_process_requests : updating embed : ".$req->embed." value: ".$req->value);
                
                $query = "UPDATE ".EMB_TABLE." SET value = '$req->value', description = '$req->description' " 
                                  ."WHERE embed = '$req->embed'";
                $rc = $wpdb->query($query);
                $msgno = 1;
                emb_trace("fn:emb_process_requests : rc = ".$rc." query = ".$query);
                
            } else if ($req->action == 'embed-add-entry') {

                if (emb_validate_embed_name($req->embed, $req)) {
                	$msgno = 2;            	
                    // Create a new index and index page using the settings provided by the user.
                    emb_trace("fn:emb_process_requests : adding index for embed : ".$req->embed);
                    
                    $rc = $wpdb->query("INSERT INTO ".EMB_TABLE." VALUES ('$req->embed', '$req->description', '$req->value')");

                    if (!$rc) {
                        $msgno = 21;
                    }
                }
                emb_trace("fn:emb_process_requests : embed-add-entry : rc = ".$rc."");
                
            } else if ($req->action == 'embed-rename-entry') {

                $newembed = $req->embed;
                $count = $req->set_vars_from_table($req->oldembed);

                if (emb_validate_embed_name($newembed, $req)) {
                    $msgno = 4;            	
                    // Create a new entry based on the entry being renamed and then delete the old one.
                    emb_trace("fn:emb_process_requests : renaming embed : from: ".$req->oldembed.", to: ".$req->embed);

                    $rc = $wpdb->query("INSERT INTO ".EMB_TABLE." VALUES ('$newembed', '$req->description', '$req->value')");

                    if (!$rc) {
                        $msgno = 22;
                    } else {
                        $rc = $wpdb->query("DELETE FROM ".EMB_TABLE." WHERE embed = '$req->oldembed'");
                        $msgno = $rc ? 4 : 22;
                    }
                }

                emb_trace("fn:emb_process_requests : embed-rename-entry : rc = ".$rc."");

            } else if ($req->action == 'embed-copy-entry') {

                $newembed = $req->embed;
                $count = $req->set_vars_from_table($req->oldembed);

                if (emb_validate_embed_name($newembed, $req)) {
                    $msgno = 5;            	
                    // Create a new entry based on the entry being renamed and then delete the old one.
                    emb_trace("fn:emb_process_requests : copying embed : from: ".$req->oldembed.", to: ".$req->embed);

                    $rc = $wpdb->query("INSERT INTO ".EMB_TABLE." VALUES ('$newembed', '$req->description', '$req->value')");

                    if (!$rc) {
                        $msgno = $rc ? 5 : 23;
                    }
                }
                emb_trace("fn:emb_process_requests : embed-copy-entry : rc = ".$rc."");

            } else if ($req->action == 'embed-delete-entry') {
                
            	// Delete the index selected by the user.
                emb_trace("fn:emb_process_requests : embed-delete index : ".$req->embed);
                
                $rc = $wpdb->query("DELETE FROM ".EMB_TABLE." WHERE embed = '$req->embed'");
                $msgno = $rc ? 3 : 31;

            } else if ($req->action == 'embed-uninstall-plugin') {
                emb_plugin_uninstall();
                wp_redirect('plugins.php');
                return;
            }

            if (empty($req->error_field)) {
                // Redirect the browser to the main admin index page.
                $req->action = 'embed-redirect';
                emb_redirect($msgno);
            } else {
                $req->escape_vars();
            }
        }
    } else {
        // Nothing to do but fetch the parameters from the 'GET' request. 
        $req->set_vars_from_get();   
    }
    // Save the request parameters in a global variable for later use.
    $emb_req = $req;
    emb_trace('fn:emb_process_requests : end');
}

function emb_validate_embed_name($embed, $req) {
    global $wpdb;
    $results = $wpdb->get_results("SELECT embed FROM ".EMB_TABLE." WHERE embed = '$embed'");
    if (count($results) != 0) {
        $req->error_field['embed'] = 'embed-error';
        $req->error_messages[] = "The name '$embed' is already in use. Please select another name.";
    }
    return (count($results) == 0);
}

function emb_get_embeds($fields, $sortby = false) {
    global $wpdb;
    
    $current_sort = get_option('emb_sort_index_table');
    
    if (!empty($sortby)) {
        if (strpos($current_sort, $sortby) !== false) {
            if (strpos($current_sort, "ASC") !== false) {
                $current_sort = str_replace("ASC", "DESC", $current_sort);
            } else {
                $current_sort = str_replace("DESC", "ASC", $current_sort);
            }
        } else {
            $current_sort = " ORDER BY ".$sortby." ASC";
        }
        update_option('emb_sort_index_table', $current_sort);
    }
    emb_trace("emb_get_embeds : SELECT $fields FROM ".EMB_TABLE.$current_sort);
    return $wpdb->get_results("SELECT $fields FROM ".EMB_TABLE.$current_sort);
}

function emb_get_embed($embed, $fields) {
    global $wpdb;
    return $wpdb->get_results("SELECT $fields FROM ".EMB_TABLE." WHERE embed = $embed");
}

/**
 * Make an http redirect request to put us back on the main indexes admin page.
 * 
 * @param $msg number of the message to be displayed, if non-zero. 
 */
function emb_redirect($msg) {
    emb_trace('fn:emb_redirect : redirect to '.EMB_PAGENAME.($msg != 0 ? '&msg='.$msg : ''));
    wp_redirect(EMB_PAGENAME.($msg != 0 ? '&msg='.$msg : ''));
}

/**
 * Display the manage index administration page.  The page will only
 * be displayed if the supplied action isn't 'embed-redirect'.
 */
function emb_display_admin_page() {
        
    emb_trace('--- admin --------------------------------------------');
    emb_trace('fn:emb_manage_embeds : start');
    global $emb_req;
    emb_trace('fn:emb_manage_embeds : action = '.$emb_req->action);
    
    if ($emb_req->action == 'embed-sort-entries' || empty($emb_req->action) && empty($emb_req->error_field)) {
        // By default, display the table of indexes.
        emb_display_embed_table($emb_req); 
    } else if ($emb_req->action != 'embed-redirect') {
        // Otherwise, display the edit panel for an index. 
        emb_display_embed_dialog($emb_req);
    }
    emb_trace('fn:emb_manage_embeds : end');
}

/**
 * Display the table of embeds
 *
 * @param $req request parameters used to fill in the table contents on the page
 */
function emb_display_embed_table($req) { 

    global $wpdb; 
    ?>

    <style type="text/css">
        .embed-hide {
            display: none;
        }
        .embed-error { 
            background-color: #fcc;
            color: #f00;
            font-weight: bold;          
        }
        #uninstall-button:disabled, #uninstall-button:disabled:hover {
            border-color: #ccc;
            background-color : #ddd;
            color : #fff;
        }
    </style>
    <div class="wrap">
        <?php if (emb_is_table_present() && emb_is_compatible_version()) { ?>
            <h2>Manage Embeds (<a href="<?php echo EMB_PAGENAME; ?>&amp;action=embed-new-entry">add new</a>)</h2>
            <?php if (!empty($req->info_message)) { ?>
                <div id="message" class="updated fade <?php echo ($req->info_message > 9 ? 'embed-error' : '')?>">
                    <p><?php echo emb_get_info_message($req->info_message); ?></p>
                </div>
            <?php }
            // Fetch the index definitions from the database.
            $entries = emb_get_embeds("*", $req->sortby);
            if (count($entries) > 0) { ?>
                <br style="clear" />
                <table class="widefat">
                    <thead>
                        <tr>
                            <th><a href='<?php echo EMB_PAGENAME; ?>&amp;action=embed-sort-entries&amp;by=embed' class='reset'>Name</a></th>
                            <th><a href='<?php echo EMB_PAGENAME; ?>&amp;action=embed-sort-entries&amp;by=description' class='reset'>Description</a></th>
                            <th>Attributes</th>
                            <th><a href='<?php echo EMB_PAGENAME; ?>&amp;action=embed-sort-entries&amp;by=value' class='reset'>Value</a></th>
                            <th colspan="4">Actions</th>
                        </tr>
                    </thead>
                    <tbody id="manage_embed_pages">
                        <?php foreach ($entries as $entry) { 
                            // Decode for Javascript function call 
                            $name = addslashes(html_entity_decode($entry->embed));

                            // Truncate the length of the value of the embed to 80 characters
                            if (strlen($entry->value) > 150) {
                                $entry->value = substr($entry->value, 0, 150).'...';
                            }
                            ?> 
                            <tr>
                                <td><?php echo $entry->embed; ?></td>
                                <td><?php echo $entry->description; ?></td>
                                <td><?php echo emb_get_attributes($entry->value); ?></td>
                                <td><?php echo $entry->value; ?></td>
                                <td><a href='<?php echo EMB_PAGENAME; ?>&amp;action=embed-edit-entry&amp;embed=<?php echo $entry->embed; ?>' class='edit'>Edit</a></td>
                                <td><a href='<?php echo EMB_PAGENAME; ?>&amp;action=embed-newname-entry&amp;oldembed=<?php echo $entry->embed; ?>' class='edit'>Rename</a></td>
                                <td><a href='<?php echo EMB_PAGENAME; ?>&amp;action=embed-newcopy-entry&amp;oldembed=<?php echo $entry->embed; ?>' class='edit'>Copy</a></td>
                                <td><a href='<?php echo EMB_PAGENAME; ?>&amp;action=embed-delete-entry&amp;embed=<?php echo $entry->embed; ?>' onclick='return delete_embed(<?php echo "\"".$name."\""; ?>)' class="reset">Delete</a></td>
                            </tr>
                        <?php } ?>
                    </tbody>
                </table>
            <?php } else { ?>
                <p>No embeds found.</p>
            <?php } 
        } else { ?>
            <div id="message" class="error fade embed-error"><p>
                <?php if (!emb_is_table_present()) {
                    echo emb_get_admin_error_message('database-not-found'); 
                } else {
                    echo emb_get_admin_error_message('version-incompatible'); 
                } ?>
            </p></div>
        <?php } ?>
        <script type="text/javascript">
            function delete_embed(name) {
                return confirm('Are you sure you want to delete the embed "'+name+'"?');
            }
            function uninstall_plugin() {
                link = document.getElementById('uninstall');
                link.className = link.className == '' ? 'embed-hide' : ''; 
                document.getElementById('uninstall-check').checked = false;
                document.getElementById('uninstall-button').disabled = true;
            }
            function uninstall_confirmed() {
                disable = !document.getElementById('uninstall-check').checked;
                document.getElementById('uninstall-button').disabled = disable;
            }
        </script>
        <div style="margin-top:20px;border:2px solid #ffcc99;background-color:#ffeecc;padding:10px;">
             For more help with Embedder, please visit the 
             <a href="http://embedder.englishmike.net">Embedder Plugin Website</a>
             where you will find tutorials, examples, and plenty of other information
             on using the plugin. 
        </div>
        <div class="tablenav" />
        <br class="clear" />
    </div>
    <?php emb_display_uninstall_form(); ?>
    <br/>
</div>  <!-- Extra end-div pushes footer to bottom of screen. -->
<?php }

/**
 * Display the uninstall form in the current admin page.
 */
function emb_display_uninstall_form() {  ?>
    <br/>
    <a style="font-size:8pt" href="#" onclick='uninstall_plugin()'>Uninstall Embedder Plugin</a>
    <div id='uninstall' class='embed-hide'>
        <p>You are about to uninstall the Embedder plugin.
        <p>All embeds and settings created using the Embedder plugin will be deleted, including the database table used by the plugin (wp_emb_embeds).</p>
        <p>If you just want to deactivate the plugin temporarily, go to the <a href="plugins.php">Plugin Management</a> page and deactivate it from there</p>  
        <p style="color:red">Once you click uninstall, it cannot be undone.</p>
        <p>Note: this will not delete the plugin from the system. The plugin can still be reactivated from the Plugin Management page later.</p>
        <div style="margin:20px 0 0 40px;padding:30px;text-align:center;width:500px;border 1px dotted #ddd;background-color:#eee;">
            <p>Are you sure you want to uninstall the Embedder plugin?</p>
            <input type="checkbox" name="uninstall-check" id="uninstall-check" onclick='uninstall_confirmed()'>&nbsp;Yes 
            <br/>
            <form name="uninstall" id="uninstall" class="add validate" method="post" action="<?php echo EMB_PAGENAME; ?>">
                <input type="hidden" name="action" value="embed-uninstall-plugin" />
                <input style="margin-top: 20px;" disabled id="uninstall-button" type="submit" class="button" name="submit" value="Uninstall Embedder" />
            </form>
        </div>  
    </div>
<?php }


function emb_get_attributes($value) {
    $output = '';
    $count = preg_match_all('/%[\w-]+%/', $value, $matches);
    $output .= implode(', ', array_unique($matches[0]));
    return strtolower(str_replace('%', '', $output));
}

/**
 * Display the add/edit index dialog, and set the fields specified in
 * $req if necessary. 
 *
 * @param $req parameters to be used in the dialog fields
 */
function emb_display_embed_dialog($req) { 

    $add = $req->action == 'embed-add-entry' || $req->action == 'embed-new-entry';
    $edit = $req->action == 'embed-edit-entry' || $req->action == 'embed-update-entry';
    $rename = $req->action == 'embed-newname-entry' || $req->action == 'embed-rename-entry';
    $copy = $req->action == 'embed-newcopy-entry' || $req->action == 'embed-copy-entry';

    ?>
    <style type="text/css">
        .embed-error { 
            background-color: #fcc;                 
        }
        .embed-hide {
            display: none;
        }
        .embed-disabled {
            color: #AAA;
        }
        #embed-form-table th {
            font-weight:bold;
            width:150px;
            padding-top:6px;
        }
    </style>
    <div class="wrap">
        <?php
        if (!empty($req->error_messages)) { ?>
            <div id="message" class="updated fade"><h3>Error message(s):</h3>
                <ul>
                    <?php foreach ($req->error_messages as $message) {
                       echo '- '.$message."<br/>";
                    } ?>
                </ul>
                <p>Please correct the highlighted field(s).</p>
            </div>
        <?php }
        if ($edit) {
            echo '<h2>Edit Embed</h2>';
        } else if ($rename) {
            echo '<h2>Rename Embed</h2>';
        } else if ($copy) {
            echo '<h2>Copy Embed</h2>';
        } else {
            echo '<h2>Add Embed</h2>';
        } ?>
            
        <div id="ajax-response"></div>
        <span id="autosave"></span>
        <form name="addembed" id="addembed" class="add validate" method="post" action="<?php echo EMB_PAGENAME; ?>">
            <table id="embed-form-table" class="form-table">
                <?php if ($rename || $copy) { ?>
                    <tr class="form-field form-required">
                        <th scope="row"><label for="embed">Old embed name</label></th>
                        <td>
                            <?php echo $req->oldembed; ?>
                        </td>
                    </tr>
                <?php } ?>
                <tr class="form-field form-required">
                    <th scope="row">
                        <?php if ($rename || $copy) { ?>
                            <label for="embed">New embed name</label>
                        <?php } else { ?>
                            <label for="embed">Embed name</label>
                        <?php } ?>
                    </th>
                    <td class="<?php echo $req->error_field['embed']; ?>" >
                        <?php if ($add || $rename || $copy) { ?>
                            <input name="embed" id="embed" type="text" value="<?php echo $req->embed; ?>" size="20" />
                            <br/>This is the name you will use wherever you want to embed the HMTL for this embed.
                        <?php } else { ?>
                            <?php echo $req->embed; ?>
                        <?php } ?>
                    </td>
                </tr>
                <?php if ($add || $edit) { ?>
                    <tr class="form-field" id="description-row">
                        <th scope="row"><label for="description" id="description-head">Description</label></th>  
                        <td>
                            <input name="description" id="description" type="text" value="<?php echo $req->description; ?>" size="40" />
                            <br/>Enter a description of the embed to remind you of what it represents.
                        </td>
                    </tr>
                    <tr class="form-field" id="value-row">
                        <th scope="row"><label for="value" id="value-head">Value</label></th>  
                        <td class="<?php echo $req->error_field['value']; ?>">
                            <textarea id="value" name="value" rows="8" cols="80"><?php echo $req->value; ?></textarea>
                            <br/>Enter the HTML you want to replace the embed with.
                        </td>
                    </tr>
                <?php } ?>
            </table>
            <p class="submit">
                <?php if ($edit) { ?>
                    <?php wp_nonce_field('emb_plugin_update'); ?>
                    <input type="hidden" name="embed" value="<?php echo $req->embed; ?>" />
                    <input type="hidden" name="action" value="embed-update-entry" />
                    <input type="submit" class="button" name="submit" value="Save Changes" />&nbsp;
                    <input type="submit" class="button" name="cancel" value="Cancel" />
                <?php } else if ($rename) { ?>
                    <input type="hidden" name="oldembed" value="<?php echo $req->oldembed; ?>" />
                    <input type="hidden" name="action" value="embed-rename-entry" />
                    <input type="submit" class="button" name="submit" value="Rename" />&nbsp;
                    <input type="submit" class="button" name="cancel" value="Cancel" />
                <?php } else if ($copy) { ?>
                    <input type="hidden" name="oldembed" value="<?php echo $req->oldembed; ?>" />
                    <input type="hidden" name="action" value="embed-copy-entry" />
                    <input type="submit" class="button" name="submit" value="Copy" />&nbsp;
                    <input type="submit" class="button" name="cancel" value="Cancel" />
                <?php } else { ?>                    
                    <?php wp_nonce_field('emb_plugin_add'); ?>
                    <input type="hidden" name="action" value="embed-add-entry" />
                    <input type="submit" class="button" name="submit" value="Add Embed" />&nbsp;
                    <input type="submit" class="button" name="cancel" value="Cancel" />
                <?php } ?>                    
            </p>
        </form>
        <script type="text/javascript">
        
            // This function controls the hiding/showing, enabling/disabling
            // of selected items in the admin panel. 
            function selchanged(field, value) {
            }
        </script>
    </div>
<?php }

/**
 * Add an option field to one of the index heading/description selection lists.
 *
 * @param $name name of option field
 * @param $title title of option filed
 * @param $sel true if item is to be selected
 */
function emb_add_option($name, $title, $sel) {
    echo '<option value="'.$name.'" '.($sel ? 'selected ' : '').'>'.$title.'</option>';
}

/**
 * Generate an <option> field, and set to checked if necessary.
 * Also adds javascript to select items in order to dynamically show/enable related fields.
 *
 * @param $value value of the option field
 * @param $sel set to checked if option is contained within this string.
 * @return string to the included in the option tag
 */
function emb_check_value($value, $sel) {
   $output = 'name="options[]" id="'.$value.'" value="'.$value.'" '.(strpos($sel, $value) !== false ? 'checked ' : '');
   return $output;
}

/**
 * Get the information message to display (if any).
 */
function emb_get_info_message($id) {
    switch ($id) {
        case 1: $message = 'Embed updated successfully.'; break;
        case 2: $message = 'Embed added successfully.'; break;
        case 3: $message = 'Embed deleted successfully.'; break;
        case 4: $message = 'Embed renamed successfully.'; break;
        case 5: $message = 'Embed copied successfully.'; break;
        case 11: $message = 'ERROR: Unable to update the embed.'; break;
        case 21: $message = 'ERROR: Unable to add the new embed.'; break;
        case 22: $message = 'ERROR: Unable to rename the embed.'; break;
        case 23: $message = 'ERROR: Unable to copy the embed.'; break;
        case 31: $message = 'ERROR: Unable to delete the embed.'; break;
    }
    return $message;
}

/**
 * Display serious error messages to the user.
 *
 * @param string $id the id of the error message to display.
 * @return none
 */
function emb_get_admin_error_message($id) {
    global $wpdb;
    $error = 'EMBEDDABLE_PLUGIN_ERROR: ';
    switch ($id) {
        case 'database-not-found': 
            $error .= '101 - The required database table - '.EMB_TABLE.' - does not exist. '
                     .'Please deactivate then re-activate the Embedder plugin to correct the problem.';
                     break;
        case 'version-out-of-date': 
            $error .= '102 - The database table is out-of-date with this version of the plugin. '  
                     .'Please deactivate then re-activate the Embedder plugin to upgrade the database. '
                     .'All existing index settings in the Embedder table will be saved.';
            break;
        case 'version-incompatible': 
            $error .= '103 - The database table is incompatible with this version of the plugin. '  
                     .'Please UNINSTALL the plugin using the link below then reactive the Embedder plugin to rectify the problem. '
                     .'All existing index settings in the Embedder table will be lost.';
            break;
        case 'rename-table-failed': 
            $error .= '201 - The Embedder plugin was unable to finish upgrading your existing embedder settings. ';  
            break;
        case 'create-table-failed': 
            $error .= '202 - The Embedder plugin was unable to create the new settings database table while upgrading to the latest version. ';
            break;  
        case 'fresh-create-table-failed': 
            $error .= '203 - The Embedder plugin was unable to create the new settings database table.';  
            break;
        case 'upgrade-table-failed':
            $error .= '204 - The Embedder plugin was unable to upgrade the new settings while upgrading to the latest version. ';
            break;  
    }
    
    // If this is an upgrade error, then display more information to help assure the user 
    // that no data has been lost, and to point them to somewhere where they can get some help.
    if (strpos($id, 'table-failed') !== false) {
        $error = '<p>'.$error.'</p>'
                .(strpos($id, 'fresh') === false ? '<p>All your existing settings have been saved, but you will not be able to use the new version of Embedder plugin. '
                .'If you want to continue running Embedder, please reinstall the <a href="http://wordpress.org/extend/plugins/embedder/download/">previous version of Embedder</a> '
                .'you were running on this blog.</p>' : '')
                .'<p>For more assistance please visit to the <a href="http://embedder.englishmike.net">Embedder Plugin Home Page</a>. '
                .'or contact the plugin author at <a href="mailto:embedder@englishmike.net">embedder@englishmike.net</a> with this information.</p>'
                .'<p>WordPress version = '.get_bloginfo('version')
                .'<br/>Database version = '.$wpdb->db_version()
                .'<br/>PHP version = '.phpversion()
                .'<br/>OS version = '.PHP_OS.'</p>';
    }
    return $error;
}

/**
 * Action hook used to display an error message on the plugins page 
 * if something has gone wrong with the database during the install or 
 * upgrade.
 */
function emb_admin_notices() {

    $error = get_option('emb_plugin_error');
    if (!empty($error)) {
        $message = emb_get_admin_error_message($error);
    }

    if (!empty($message)) {
        echo '<div class="error" style="padding:10px 10px">';
        echo '<strong>Fatal Embedder Plugin Upgrade Error</strong>';
        echo '<p>'.$message.'</p>';
        
        if (function_exists('deactivate_plugins') ) {
            deactivate_plugins( EMB_PLUGIN_FILE);
            echo '<p>The Embedder plugin has been deactivated.</p>';
        }
        echo '</div>';
    }
    
    // Remember to delete the option so the message is not shown again.
    delete_option('emb_plugin_error');
}


/**
 * htmlspecialchars_decode is a >= PHP5.1-only function.  This version of the function 
 * will allow the plugin to work on PHP4 installations. 
 */
function emb_htmlspecialchars_decode($string, $style=ENT_COMPAT) {
    if (floatval(phpversion()) >= 5.1) {
        $string = htmlspecialchars_decode($string, $style); 
    } else {
        $translation = array_flip(get_html_translation_table(HTML_SPECIALCHARS,$style));
        if ($style === ENT_QUOTES) { 
            $translation['&#039;'] = '\''; 
        }
        $string = strtr($string,$translation);
    }     
    return $string;  
}

/**
 * Class containing all the data sent in a POST or GET message or
 * an entry retrieved from the indexes database.
 */
class emb_request { 
    var $action;       // Current action to be processed
    var $sortby;       // Set if the embed table is to be sorted
    var $embed;        // Name of the embed
    var $description;  // Description of the embed
    var $value;        // Value of the embed
    var $oldembed;     // Old name of the embed during rename operation
    var $info_message; // Informational message to the output to the user
    var $error_field = array();    // Input fields currently in error 
    var $error_messages = array(); // Error message to be output to the admin user

    /**
     * Constructor for class. Initialize the action instance variable immediately. 
     */
    function emb_request() {
        emb_trace('fn:emb_request');
        $this->action = $_GET['action'];
        if (empty($this->action)) {
            if (empty($_POST['cancel'])) {
                $this->action = $_POST['action'];
            } else {
                $this->action = 'cancel';
            }
        }
    }
    
    /**
     * Populate the instance data for the class using the GET request parameters.
     * Also generate the appropriate error messages should problems be found.
     */
    function set_vars_from_post() {
        global $wpdb;
        emb_trace('fn:set_vars_from_post');
        $this->embed = strtolower(stripslashes(trim($_POST['embed'])));
        $this->description = stripslashes(trim($_POST['description']));
        $this->value = htmlentities(stripslashes(trim($_POST['value'])));
        $this->oldembed = stripslashes(trim($_POST['oldembed']));
        $this->options = !empty($_POST['options']) ? implode(',', $_POST['options']) : '';
        $this->info_message = intval($_POST['msg']);

        if (empty($this->embed)) {
            $this->error_field['embed'] = 'embed-error';
            $this->error_messages[] = 'A name for the embed must be specified.';
        } else {
            if (preg_match('/^[\w-]+$/', $this->embed) == 0) {
                $this->error_field['embed'] = 'embed-error';
                $this->error_messages[] = 'Invalid characters found in the embed '.$this->embed;
            }
        }
        if ($this->action != 'embed-rename-entry' && $this->action != 'embed-copy-entry' && empty($this->value)) {
            $this->error_field['value'] = 'embed-error';
            $this->error_messages[] = 'A value for this embed must be specified.';
        }
    }

    /**
     * Populate the instance data for the class using the GET request parameters
     * and, where necessary, the selected entry in the index database table.
     */
    function set_vars_from_get() {
        emb_trace('fn:set_vars_from_get');
        $this->embed = $_GET['embed'];
        $this->info_message = intval($_GET['msg']);
        $this->oldembed = $_GET['oldembed'];
        
        if ($this->action == 'embed-edit-entry') {
            $this->set_vars_from_table($this->embed);
        } else if ($this->action == 'embed-sort-entries') {
            $this->sortby = $_GET['by'];
        } 
    }

    /**
     * Fetch the settings for the current index from the database.
     *
     * @param $id id of index page
     * @return number of indexes found (i.e. if zero, then there is no index) 
     */
    function set_vars_from_table($embed) {
        emb_trace('fn:set_vars_from_table: '.$embed);
        global $wpdb;
        $entries = $wpdb->get_results("SELECT * FROM ".EMB_TABLE." WHERE embed = '$embed'");
        if (count($entries) != 0) {
            $this->set_vars_from_index($entries[0]);
        }
        return count($entries);
    }
    /**
     * Fetch the settings for the supplied index object
     *
     * @param $id id of index page
     * @return number of indexes found (i.e. if zero, then there is no index) 
     */
    function set_vars_from_index($entry) {
        emb_trace('fn:set_vars_from_index');
        $this->embed = $entry->embed;
        $this->description = $entry->description;
        $this->value = $entry->value;
        $this->options = $entry->options;
    }
    
    /**
     * Escape all the strings that might contain illegal characters,
     * ready for insertion into the wordpress database.
     */
    function escape_vars() {
        $this->embed = $this->embed;
        $this->description = $this->description;
        $this->value = $this->value;
    }
}

/**
 * Helper string function.
 *
 * @param $str haystack
 * @param $sub needle
 * @return true if the haystack starts with the needle
 */
function emb_begins_with($str, $sub) {
    return substr($str, 0, strlen($sub)) == $sub;
}

/**
 * Helper string function.
 *
 * @param $str haystack
 * @param $sub needle
 * @return true if the haystack ends with the needle
 */
function emb_ends_with($str, $sub) {
    return substr($str, strlen($str) - strlen($sub)) == $sub;
}

/**
 * DEBUG function to send trace output to a trace file on the server.
 * For debugging purposes only.
 *
 * @param $output output to be sent to trace file
 */
function emb_trace($output, $array = 0) {
    if (EMB_DEBUG) {
        //emb_println($output);
        $handle = fopen(ABSPATH.'trace', 'a+');
        fwrite($handle, $output);
        if (!empty($array)) {
            fwrite($handle, $implode(", ", $array));
        }
        fwrite($handle, "\n");
        fclose($handle);
    }
}

/**
 * DEBUG function for println out information on the web page. 
 * Does not work in cases where the webpage is not being generated. 
 *
 * @param $value string to be output.
 */
function emb_println($value) {
    echo '<span style="color:red;background:yellow">'.$value.'</span><br/>';
}

function emb_print_r($var) {
    echo str_replace(chr(10), '<br>', print_r($var, true));
}

function emb_print_t($var) {
    emb_trace(print_r($var, true));
}

function emb_add_help_text($contextual_help, $screen_id, $screen) { 
  if ($screen->id == EMB_PAGEPREFIX.EMB_PAGEID) {
      $action = $_GET['action'];
      switch ($action) {
          case "embed-edit-entry":
              $contextual_help = "HELP MAIN PAGE";
              //    ."<p></p>"
              //    ."<p></p>"
              break;
          default:
              $contextual_help = "<p>This is the main settings page for the Embedder plugin, which allows you to create HTML embeds to insert into your blog posts and pages.</p>"
                                ."<p>This page displays a table of all the embeds you have created.</p>"
                                ."<p>For each embed, the table displays its name, a description, its value, and any attributes the value contains.</p>"
                                ."<p>Use the links on the right-hand side of the table to <b>Edit</b>, <b>Rename</b>, <b>Copy</b>, or <b>Delete</b> an embed.</p>"
                                ."<p>To create a new embed, click the <b><i>(add new)</i></b> link at the top of the page.</p>"
                                ."<p>The table can be sorted by name, description, or value. Click on the title of the appropriate column to sort the table.</p>"
                                ."<p>Once you have created an embed, you can use it in any post or page by using the syntax: <b>[embed-name]</b></p>"
                                ."<p>For example, if you want to embed a Google map in a post, you can create an embed called <b>'google-map'</b> containing the HTML copied from Google Maps, "
                                ."then put the following in the text of your blog post:</p>"
                                ."<p style='padding-left:20px'><b>[google-map]</b></p>"
                                ."<p>For more flexibility, you can add one or more attributes to an embed by inserting the name of the attribute like this: <b>%attribute-name%</b>.</p>"
                                ."<p>For example, if you want to play with the width and height of the Google map in the above example, you can replace the fixed width and height "
                                ."in the HTML with %width% and %height%. Then in your post you can put this:</p>"
                                ."<p style='padding-left:20px'><b>[google-map width='500' height='400']</b></p>"
                                ."<p>Finally, if you want to wrap some HTML around some text, you can create an embed with the attribute %content% as the place holder for the text.</p>"
                                ."<p>For example, if you want to float an image to the right-hand side of a post, you can create an embed called <b>'float-right'</b> with the following HTML:</p>"
                                ."<p style='padding-left:20px'><b>&lt;div style='padding:2px;border:1px solid gray'&gt;&lt;img src='%content%'&gt;&lt;/div&gt;</b></p>"
                                ."<p>and then you can use it to float any image of your choice in any post or page like this:</p>"
                                ."<p style='padding-left:20px'><b>[float-right]http://www.images.com/myimage.jpg[/float-right]</b></p>";
              break;
      }
  }
  $contextual_help .= var_dump($screen); // use this to help determine $screen->id
  $contextual_help .= var_dump($screen_id); // use this to help determine $screen->id
  return $contextual_help;
}


?>